@@ -51,12 +51,17 @@ def _invalid_netcdf_feature(feature, allow):
 
 def _transform_1d_boolean_indexers(key):
     """Find and transform 1D boolean indexers to int"""
-    key = [
-        np.asanyarray(k).nonzero()[0]
-        if isinstance(k, (np.ndarray, list)) and type(k[0]) in (bool, np.bool_)
-        else k
-        for k in key
-    ]
+    # return key, if not iterable
+    try:
+        key = [
+            np.asanyarray(k).nonzero()[0]
+            if isinstance(k, (np.ndarray, list)) and type(k[0]) in (bool, np.bool_)
+            else k
+            for k in key
+        ]
+    except TypeError:
+        return key
+
     return tuple(key)
 
 
@@ -145,10 +150,14 @@ class BaseVariable(object):
         # normal variable carrying DIMENSION_LIST
         # extract hdf5 file references and get objects name
         if "DIMENSION_LIST" in attrs:
-            return tuple(
-                self._root._h5file[ref[0]].name.split("/")[-1]
-                for ref in list(self._h5ds.attrs.get("DIMENSION_LIST", []))
-            )
+            # check if malformed variable and raise
+            if _unlabeled_dimension_mix(self._h5ds) == "labeled":
+                # If a dimension has attached more than one scale for some reason, then
+                # take the last one. This is in line with netcdf-c and netcdf4-python.
+                return tuple(
+                    self._root._h5file[ref[-1]].name.split("/")[-1]
+                    for ref in list(self._h5ds.attrs.get("DIMENSION_LIST", []))
+                )
 
         # need to use the h5ds name here to distinguish from collision dimensions
         child_name = self._h5ds.name.split("/")[-1]
@@ -271,6 +280,39 @@ class BaseVariable(object):
         """Return NumPy dtype object giving the variable’s type."""
         return self._h5ds.dtype
 
+    def _get_padding(self, key):
+        """Return padding if needed, defaults to False."""
+        padding = False
+        if self.dtype != str and self.dtype.kind in ["f", "i", "u"]:
+            key0 = _expanded_indexer(key, self.ndim)
+            key0 = _transform_1d_boolean_indexers(key0)
+            # extract max shape of key vs hdf5-shape
+            h5ds_shape = self._h5ds.shape
+            shape = self.shape
+
+            # check for ndarray and list
+            # see https://github.com/pydata/xarray/issues/7154
+            # first get maximum index
+            max_index = [
+                max(k) + 1 if isinstance(k, (np.ndarray, list)) else k.stop
+                for k in key0
+            ]
+            # second convert to max shape
+            max_shape = tuple(
+                [
+                    shape[i] if k is None else max(h5ds_shape[i], k)
+                    for i, k in enumerate(max_index)
+                ]
+            )
+
+            # check if hdf5 dataset dimensions are smaller than
+            # their respective netcdf dimensions
+            sdiff = [d0 - d1 for d0, d1 in zip(max_shape, h5ds_shape)]
+            # create padding only if hdf5 dataset is smaller than netcdf dimension
+            if sum(sdiff):
+                padding = [(0, s) for s in sdiff]
+        return padding
+
     def __array__(self, *args, **kwargs):
         return self._h5ds.__array__(*args, **kwargs)
 
@@ -279,7 +321,6 @@ class BaseVariable(object):
 
         if isinstance(self._parent._root, Dataset):
             # this is only for legacyapi
-            key = _expanded_indexer(key, self.ndim)
             # fix boolean indexing for affected versions
             # https://github.com/h5py/h5py/pull/2079
             # https://github.com/h5netcdf/h5netcdf/pull/125/
@@ -292,18 +333,17 @@ class BaseVariable(object):
             if string_info and string_info.length is None:
                 return self._h5ds.asstr()[key]
 
-        # return array padded with fillvalue (both api)
-        if self.dtype != str and self.dtype.kind in ["f", "i", "u"]:
-            sdiff = [d0 - d1 for d0, d1 in zip(self.shape, self._h5ds.shape)]
-            if sum(sdiff):
-                fv = self.dtype.type(self._h5ds.fillvalue)
-                padding = [(0, s) for s in sdiff]
-                return np.pad(
-                    self._h5ds,
-                    pad_width=padding,
-                    mode="constant",
-                    constant_values=fv,
-                )[key]
+        # get padding
+        padding = self._get_padding(key)
+        # apply padding with fillvalue (both api)
+        if padding:
+            fv = self.dtype.type(self._h5ds.fillvalue)
+            return np.pad(
+                self._h5ds,
+                pad_width=padding,
+                mode="constant",
+                constant_values=fv,
+            )[key]
 
         return self._h5ds[key]
 
@@ -406,15 +446,26 @@ def _netcdf_dimension_but_not_variable(h5py_dataset):
 
 
 def _unlabeled_dimension_mix(h5py_dataset):
-    dims = sum([len(j) for j in h5py_dataset.dims])
-    if dims:
-        if dims != h5py_dataset.ndim:
+    # check if dataset has dims and get it
+    dimlist = getattr(h5py_dataset, "dims", [])
+    if not dimlist:
+        status = "nodim"
+    else:
+        dimset = set([len(j) for j in dimlist])
+        # either all dimensions have exactly one scale
+        # or all dimensions have no scale
+        if dimset ^ {0} == set():
+            status = "unlabeled"
+        elif dimset & {0}:
             name = h5py_dataset.name.split("/")[-1]
             raise ValueError(
                 "malformed variable {0} has mixing of labeled and "
                 "unlabeled dimensions.".format(name)
             )
-    return dims
+        else:
+            status = "labeled"
+
+    return status
 
 
 class Group(Mapping):
@@ -462,9 +513,8 @@ class Group(Mapping):
                     self._dimensions.add(k)
                 else:
                     if self._root._phony_dims_mode is not None:
-
-                        # check if malformed variable
-                        if not _unlabeled_dimension_mix(v):
+                        # check if malformed variable and raise
+                        if _unlabeled_dimension_mix(v) == "unlabeled":
                             # if unscaled variable, get phony dimensions
                             phony_dims |= Counter(v.shape)
 
@@ -486,7 +536,7 @@ class Group(Mapping):
                     if self._root._phony_dims_mode == "sort":
                         name += self._root._max_dim_id + 1
                     name = "phony_dim_{}".format(name)
-                    self._dimensions[name] = size
+                    self._dimensions.add_phony(name, size)
 
         self._initialized = True
 
@@ -675,6 +725,14 @@ class Group(Mapping):
         if self._root._h5py.__name__ == "h5py":
             kwargs.update(dict(track_order=self._parent._track_order))
 
+        # handling default fillvalues for legacyapi
+        # see https://github.com/h5netcdf/h5netcdf/issues/182
+        from .legacyapi import Dataset, _get_default_fillvalue
+
+        fillval = fillvalue
+        if fillvalue is None and isinstance(self._parent._root, Dataset):
+            fillval = _get_default_fillvalue(dtype)
+
         # create hdf5 variable
         self._h5group.create_dataset(
             h5name,
@@ -682,7 +740,7 @@ class Group(Mapping):
             dtype=dtype,
             data=data,
             chunks=chunks,
-            fillvalue=fillvalue,
+            fillvalue=fillval,
             **kwargs,
         )
 
@@ -712,7 +770,20 @@ class Group(Mapping):
         variable._ensure_dim_id()
 
         if fillvalue is not None:
-            value = variable.dtype.type(fillvalue)
+            # trying to create correct type of fillvalue
+            if variable.dtype is str:
+                value = fillvalue
+            else:
+                string_info = self._root._h5py.check_string_dtype(variable.dtype)
+                if (
+                    string_info
+                    and string_info.length is not None
+                    and string_info.length > 1
+                ):
+                    value = fillvalue
+                else:
+                    value = variable.dtype.type(fillvalue)
+
             variable.attrs._h5attrs["_FillValue"] = value
         return variable
 
@@ -773,6 +844,12 @@ class Group(Mapping):
         """
         # if root-variable
         if name.startswith("/"):
+            # handling default fillvalues for legacyapi
+            # see https://github.com/h5netcdf/h5netcdf/issues/182
+            from .legacyapi import Dataset, _get_default_fillvalue
+
+            if fillvalue is None and isinstance(self._parent._root, Dataset):
+                fillvalue = _get_default_fillvalue(dtype)
             return self._root.create_variable(
                 name[1:],
                 dimensions,
@@ -911,6 +988,16 @@ class File(Group):
         phony_dims: 'sort', 'access'
             See :ref:`phony dims` for more details.
 
+        track_order: bool
+            Corresponds to the h5py.File `track_order` parameter. Unless
+            specified, the library will choose a default that enhances
+            compatibility with netCDF4-c. If h5py version 3.7.0 or greater is
+            installed, this parameter will be set to True by default.
+            track_order is required to be true to for netCDF4-c libraries to
+            append to a file. If an older version of h5py is detected, this
+            parameter will be set to False by default to work around a bug in
+            h5py limiting the number of attributes for a given variable.
+
         **kwargs:
             Additional keyword arguments to be passed to the ``h5py.File``
             constructor.
@@ -930,22 +1017,14 @@ class File(Group):
         # standard
         # https://github.com/Unidata/netcdf-c/issues/2054
         # https://github.com/h5netcdf/h5netcdf/issues/128
-        # 2022/01/20: hmaarrfk
-        # However, it was found that this causes issues with attrs and h5py
-        # https://github.com/h5netcdf/h5netcdf/issues/136
-        # https://github.com/h5py/h5py/issues/1385
-        track_order = kwargs.pop("track_order", False)
-
-        # When the issues with track_order in h5py are resolved, we
-        # can consider uncommenting the code below
-        # if not track_order:
-        #     self._closed = True
-        #     raise ValueError(
-        #         f"track_order, if specified must be set to to True (got {track_order})"
-        #         "to conform to the netCDF4 file format. Please see "
-        #         "https://github.com/h5netcdf/h5netcdf/issues/130 "
-        #         "for more details."
-        #     )
+        # h5py versions less than 3.7.0 had a bug that limited the number of
+        # attributes when track_order was set to true by default.
+        # However, setting track_order to True helps with compatibility
+        # with netcdf4-c and generally, keeping track of how things were added
+        # to the dataset.
+        # https://github.com/h5netcdf/h5netcdf/issues/136#issuecomment-1017457067
+        track_order_default = version.parse(h5py.__version__) >= version.parse("3.7.0")
+        track_order = kwargs.pop("track_order", track_order_default)
 
         if version.parse(h5py.__version__) >= version.parse("3.0.0"):
             self.decode_vlen_strings = kwargs.pop("decode_vlen_strings", None)
