@@ -677,6 +677,13 @@ def test_invalid_netcdf4(tmp_local_or_remote_netcdf):
             pass
 
 
+def test_fake_phony_dims(tmp_local_or_remote_netcdf):
+    # tests writing of dimension with phony naming scheme
+    # see https://github.com/h5netcdf/h5netcdf/issues/178
+    with h5netcdf.File(tmp_local_or_remote_netcdf, mode="w") as ds:
+        ds.dimensions["phony_dim_0"] = 3
+
+
 def check_invalid_netcdf4_mixed(var, i):
     pdim = "phony_dim_{}".format(i)
     assert var["foo1"].dimensions[0] == "y1"
@@ -760,9 +767,15 @@ def test_invalid_netcdf_malformed_dimension_scales(tmp_local_or_remote_netcdf):
             f["z"].make_scale()
         f["foo1"].dims[0].attach_scale(f["x"])
 
+    with raises(ValueError):
+        with h5netcdf.File(tmp_local_or_remote_netcdf, "r") as ds:
+            assert ds
+            print(ds)
+
     with raises(ValueError):
         with h5netcdf.File(tmp_local_or_remote_netcdf, "r", phony_dims="sort") as ds:
             assert ds
+            print(ds)
 
 
 def test_hierarchical_access_auto_create(tmp_local_or_remote_netcdf):
@@ -1142,6 +1155,7 @@ def test_reading_unlimited_dimensions_created_with_c_api(tmp_local_netcdf):
         assert f["dummy2"].shape == (3, 2, 2)
         f.groups["test"]["dummy3"].shape == (3, 3)
         f.groups["test"]["dummy4"].shape == (0, 0)
+        assert f["dummy5"].shape == (2, 3)
 
 
 def test_reading_unused_unlimited_dimension(tmp_local_or_remote_netcdf):
@@ -1163,10 +1177,12 @@ def test_reading_special_datatype_created_with_c_api(tmp_local_netcdf):
 
 
 def test_nc4_non_coord(tmp_local_netcdf):
-    # Track order True is the new default for versions after 0.12.0
-    # 0.12.0 defaults to `track_order=False`
-    # Ensure that the tests order the variables in their creation order
-    # not alphabetical order
+    # Here we generate a few variables and coordinates
+    # The default should be to track the order of creation
+    # Thus, on reopening the file, the order in which
+    # the variables are listed should be maintained
+    # y   --   refers to the coordinate y
+    # _nc4_non_coord_y  --  refers to the data y
     with h5netcdf.File(tmp_local_netcdf, "w") as f:
         f.dimensions = {"x": None, "y": 2}
         f.create_variable("test", dimensions=("x",), dtype=np.int64)
@@ -1177,8 +1193,23 @@ def test_nc4_non_coord(tmp_local_netcdf):
         assert f.dimensions["x"].size == 0
         assert f.dimensions["x"].isunlimited()
         assert f.dimensions["y"].size == 2
-        assert list(f.variables) == ["y", "test"]
-        assert list(f._h5group.keys()) == ["_nc4_non_coord_y", "test", "x", "y"]
+        if version.parse(h5py.__version__) >= version.parse("3.7.0"):
+            assert list(f.variables) == ["test", "y"]
+            assert list(f._h5group.keys()) == ["x", "y", "test", "_nc4_non_coord_y"]
+
+    with h5netcdf.File(tmp_local_netcdf, "w") as f:
+        f.dimensions = {"x": None, "y": 2}
+        f.create_variable("y", dimensions=("x",), dtype=np.int64)
+        f.create_variable("test", dimensions=("x",), dtype=np.int64)
+
+    with h5netcdf.File(tmp_local_netcdf, "r") as f:
+        assert list(f.dimensions) == ["x", "y"]
+        assert f.dimensions["x"].size == 0
+        assert f.dimensions["x"].isunlimited()
+        assert f.dimensions["y"].size == 2
+        if version.parse(h5py.__version__) >= version.parse("3.7.0"):
+            assert list(f.variables) == ["y", "test"]
+            assert list(f._h5group.keys()) == ["x", "y", "_nc4_non_coord_y", "test"]
 
 
 def test_overwrite_existing_file(tmp_local_netcdf):
@@ -1472,6 +1503,9 @@ def test_no_circular_references(tmp_local_netcdf):
 
 
 def test_expanded_variables_netcdf4(tmp_local_netcdf, netcdf_write_module):
+    # partially reimplemented due to performance reason in edge cases
+    # https://github.com/h5netcdf/h5netcdf/issues/182
+
     with netcdf_write_module.Dataset(tmp_local_netcdf, "w") as ds:
         f = ds.createGroup("test")
         f.createDimension("x", None)
@@ -1483,8 +1517,8 @@ def test_expanded_variables_netcdf4(tmp_local_netcdf, netcdf_write_module):
         dummy4 = f.createVariable("dummy4", float, ("x", "y"))
 
         dummy1[:] = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
-        dummy2[:] = [[1, 2, 3]]
-        dummy3[:] = [[1, 2, 3], [4, 5, 6]]
+        dummy2[1, :] = [4, 5, 6]
+        dummy3[0:2, :] = [[1, 2, 3], [4, 5, 6]]
 
         # don't mask, since h5netcdf doesn't do masking
         if netcdf_write_module == netCDF4:
@@ -1503,10 +1537,16 @@ def test_expanded_variables_netcdf4(tmp_local_netcdf, netcdf_write_module):
         f = ds["test"]
 
         np.testing.assert_allclose(f.variables["dummy1"][:], res1)
+        np.testing.assert_allclose(f.variables["dummy1"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy1"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy1"].shape == (3, 3)
         np.testing.assert_allclose(f.variables["dummy2"][:], res2)
+        np.testing.assert_allclose(f.variables["dummy2"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy2"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy2"].shape == (3, 3)
         np.testing.assert_allclose(f.variables["dummy3"][:], res3)
+        np.testing.assert_allclose(f.variables["dummy3"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy3"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy3"].shape == (3, 3)
         np.testing.assert_allclose(f.variables["dummy4"][:], res4)
         assert f.variables["dummy4"].shape == (3, 3)
@@ -1514,12 +1554,22 @@ def test_expanded_variables_netcdf4(tmp_local_netcdf, netcdf_write_module):
     with legacyapi.Dataset(tmp_local_netcdf, "r") as ds:
         f = ds["test"]
         np.testing.assert_allclose(f.variables["dummy1"][:], res1)
+        np.testing.assert_allclose(f.variables["dummy1"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy1"][1:2, :], [[4.0, 5.0, 6.0]])
+        np.testing.assert_allclose(f.variables["dummy1"]._h5ds[1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(
+            f.variables["dummy1"]._h5ds[1:2, :], [[4.0, 5.0, 6.0]]
+        )
         assert f.variables["dummy1"].shape == (3, 3)
         assert f.variables["dummy1"]._h5ds.shape == (3, 3)
         np.testing.assert_allclose(f.variables["dummy2"][:], res2)
+        np.testing.assert_allclose(f.variables["dummy2"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy2"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy2"].shape == (3, 3)
-        assert f.variables["dummy2"]._h5ds.shape == (1, 3)
+        assert f.variables["dummy2"]._h5ds.shape == (2, 3)
         np.testing.assert_allclose(f.variables["dummy3"][:], res3)
+        np.testing.assert_allclose(f.variables["dummy3"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy3"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy3"].shape == (3, 3)
         assert f.variables["dummy3"]._h5ds.shape == (2, 3)
         np.testing.assert_allclose(f.variables["dummy4"][:], res4)
@@ -1529,12 +1579,19 @@ def test_expanded_variables_netcdf4(tmp_local_netcdf, netcdf_write_module):
     with h5netcdf.File(tmp_local_netcdf, "r") as ds:
         f = ds["test"]
         np.testing.assert_allclose(f.variables["dummy1"][:], res1)
+        np.testing.assert_allclose(f.variables["dummy1"][:, :], res1)
+        np.testing.assert_allclose(f.variables["dummy1"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy1"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy1"].shape == (3, 3)
         assert f.variables["dummy1"]._h5ds.shape == (3, 3)
         np.testing.assert_allclose(f.variables["dummy2"][:], res2)
+        np.testing.assert_allclose(f.variables["dummy2"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy2"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy2"].shape == (3, 3)
-        assert f.variables["dummy2"]._h5ds.shape == (1, 3)
+        assert f.variables["dummy2"]._h5ds.shape == (2, 3)
         np.testing.assert_allclose(f.variables["dummy3"][:], res3)
+        np.testing.assert_allclose(f.variables["dummy3"][1, :], [4.0, 5.0, 6.0])
+        np.testing.assert_allclose(f.variables["dummy3"][1:2, :], [[4.0, 5.0, 6.0]])
         assert f.variables["dummy3"].shape == (3, 3)
         assert f.variables["dummy3"]._h5ds.shape == (2, 3)
         np.testing.assert_allclose(f.variables["dummy4"][:], res4)
@@ -1573,13 +1630,14 @@ def test_creation_with_h5netcdf_edit_with_netcdf4(tmp_local_netcdf):
         np.testing.assert_array_equal(variable[...].data, 10)
 
 
-# https://github.com/h5netcdf/h5netcdf/issues/136
-@pytest.mark.skip(reason="h5py bug with track_order")
-def test_track_order_false(tmp_local_netcdf):
-    # track_order must be specified as True or not specified at all
-    # https://github.com/h5netcdf/h5netcdf/issues/130
-    with pytest.raises(ValueError):
-        h5netcdf.File(tmp_local_netcdf, "w", track_order=False)
+def test_track_order_specification(tmp_local_netcdf):
+    # While netcdf4-c has historically only allowed track_order to be True
+    # There doesn't seem to be a good reason for this
+    # https://github.com/Unidata/netcdf-c/issues/2054 historically, h5netcdf
+    # has not specified this parameter (leaving it implicitely as False)
+    # We want to make sure we allow both here
+    with h5netcdf.File(tmp_local_netcdf, "w", track_order=False):
+        pass
 
     with h5netcdf.File(tmp_local_netcdf, "w", track_order=True):
         pass
@@ -1607,8 +1665,8 @@ def test_more_than_7_attr_creation_track_order(tmp_local_netcdf, track_order):
         # We don't expect any errors. This is effectively a void context manager
         expected_errors = memoryview(b"")
 
-    with expected_errors:
-        with h5netcdf.File(tmp_local_netcdf, "w", track_order=track_order) as h5file:
+    with h5netcdf.File(tmp_local_netcdf, "w", track_order=track_order) as h5file:
+        with expected_errors:
             for i in range(100):
                 h5file.attrs[f"key{i}"] = i
                 h5file.attrs[f"key{i}"] = 0
@@ -1710,6 +1768,25 @@ def test_bool_slicing_length_one_dim(tmp_local_netcdf):
             ds["hello"][bool_slice, :]
 
 
+def test_fancy_indexing(tmp_local_netcdf):
+    # regression test for https://github.com/pydata/xarray/issues/7154
+    with h5netcdf.legacyapi.Dataset(tmp_local_netcdf, "w") as ds:
+        ds.createDimension("x", None)
+        ds.createDimension("y", None)
+        ds.createVariable("hello", int, ("x", "y"), fill_value=0)
+        ds["hello"][:5, :10] = np.arange(5 * 10, dtype="int").reshape((5, 10))
+        ds.createVariable("hello2", int, ("x", "y"))
+        ds["hello2"][:10, :20] = np.arange(10 * 20, dtype="int").reshape((10, 20))
+
+    with legacyapi.Dataset(tmp_local_netcdf, "a") as ds:
+        np.testing.assert_array_equal(ds["hello"][1, [7, 8, 9]], [17, 18, 19])
+        np.testing.assert_array_equal(ds["hello"][1, [9, 10, 11]], [19, 0, 0])
+        np.testing.assert_array_equal(ds["hello"][1, slice(9, 12)], [19, 0, 0])
+        np.testing.assert_array_equal(ds["hello"][[2, 3, 4], 1], [21, 31, 41])
+        np.testing.assert_array_equal(ds["hello"][[4, 5, 6], 1], [41, 0, 0])
+        np.testing.assert_array_equal(ds["hello"][slice(4, 7), 1], [41, 0, 0])
+
+
 def test_h5py_chunking(tmp_local_netcdf):
     with h5netcdf.File(tmp_local_netcdf, "w") as ds:
         ds.dimensions = {"x": 10, "y": 10, "z": 10, "t": None}
@@ -1789,7 +1866,7 @@ def test_h5netcdf_chunking(tmp_local_netcdf):
 
 def test_create_invalid_netcdf_catch_error(tmp_local_netcdf):
     # see https://github.com/h5netcdf/h5netcdf/issues/138
-    with h5netcdf.File("test.nc", "w") as f:
+    with h5netcdf.File(tmp_local_netcdf, "w") as f:
         try:
             f.create_variable("test", ("x", "y"), data=np.ones((10, 10), dtype="bool"))
         except CompatibilityError:
@@ -1797,8 +1874,8 @@ def test_create_invalid_netcdf_catch_error(tmp_local_netcdf):
         assert repr(f.dimensions) == "<h5netcdf.Dimensions: >"
 
 
-def test_dimensions_in_parent_groups():
-    with netCDF4.Dataset("test_netcdf.nc", mode="w") as ds:
+def test_dimensions_in_parent_groups(tmpdir):
+    with netCDF4.Dataset(tmpdir.join("test_netcdf.nc"), mode="w") as ds:
         ds0 = ds
         for i in range(10):
             ds = ds.createGroup(f"group{i:02d}")
@@ -1808,7 +1885,7 @@ def test_dimensions_in_parent_groups():
         var = ds0["group00"].createVariable("x", float, ("x", "y"))
         var[:] = np.ones((10, 20))
 
-    with legacyapi.Dataset("test_legacy.nc", mode="w") as ds:
+    with legacyapi.Dataset(tmpdir.join("test_legacy.nc"), mode="w") as ds:
         ds0 = ds
         for i in range(10):
             ds = ds.createGroup(f"group{i:02d}")
@@ -1818,8 +1895,8 @@ def test_dimensions_in_parent_groups():
         var = ds0["group00"].createVariable("x", float, ("x", "y"))
         var[:] = np.ones((10, 20))
 
-    with h5netcdf.File("test_netcdf.nc", mode="r") as ds0:
-        with h5netcdf.File("test_legacy.nc", mode="r") as ds1:
+    with h5netcdf.File(tmpdir.join("test_netcdf.nc"), mode="r") as ds0:
+        with h5netcdf.File(tmpdir.join("test_legacy.nc"), mode="r") as ds1:
             assert repr(ds0.dimensions["x"]) == repr(ds1.dimensions["x"])
             assert repr(ds0.dimensions["y"]) == repr(ds1.dimensions["y"])
             assert repr(ds0["group00"]) == repr(ds1["group00"])
@@ -2025,3 +2102,78 @@ def test_array_attributes(tmp_local_netcdf):
         np.testing.assert_equal(ds.int_array, np.arange(10))
         np.testing.assert_equal(ds.empty_list, np.array([]))
         np.testing.assert_equal(ds.empty_array, np.array([]))
+
+
+@pytest.mark.skipif(
+    version.parse(h5py.__version__) < version.parse("3.7.0"),
+    reason="does not work with h5py < 3.7.0",
+)
+def test_vlen_string_dataset_fillvalue(tmp_local_netcdf, decode_vlen_strings):
+    # check _FillValue for VLEN string datasets
+    # only works for h5py >= 3.7.0
+
+    # first with new API
+    with h5netcdf.File(tmp_local_netcdf, "w") as ds:
+        ds.dimensions = {"string": 10}
+        dt0 = h5py.string_dtype()
+        fill_value0 = "bár"
+        ds.create_variable("x0", ("string",), dtype=dt0, fillvalue=fill_value0)
+        dt1 = h5py.string_dtype("ascii")
+        fill_value1 = "bar"
+        ds.create_variable("x1", ("string",), dtype=dt1, fillvalue=fill_value1)
+
+    # check, if new API can read them
+    with h5netcdf.File(tmp_local_netcdf, "r", **decode_vlen_strings) as ds:
+        decode_vlen = decode_vlen_strings["decode_vlen_strings"]
+        fvalue0 = fill_value0 if decode_vlen else fill_value0.encode("utf-8")
+        fvalue1 = fill_value1 if decode_vlen else fill_value1.encode("utf-8")
+        assert ds["x0"][0] == fvalue0
+        assert ds["x0"].attrs["_FillValue"] == fill_value0
+        assert ds["x1"][0] == fvalue1
+        assert ds["x1"].attrs["_FillValue"] == fill_value1
+
+    # check if legacyapi can read them
+    with legacyapi.Dataset(tmp_local_netcdf, "r") as ds:
+        assert ds["x0"][0] == fill_value0
+        assert ds["x0"]._FillValue == fill_value0
+        assert ds["x1"][0] == fill_value1
+        assert ds["x1"]._FillValue == fill_value1
+
+    # check if netCDF4-python can read them
+    with netCDF4.Dataset(tmp_local_netcdf, "r") as ds:
+        assert ds["x0"][0] == fill_value0
+        assert ds["x0"]._FillValue == fill_value0
+        assert ds["x1"][0] == fill_value1
+        assert ds["x1"]._FillValue == fill_value1
+
+    # second with legacyapi
+    with legacyapi.Dataset(tmp_local_netcdf, "w") as ds:
+        ds.createDimension("string", 10)
+        fill_value0 = "bár"
+        ds.createVariable("x0", str, ("string",), fill_value=fill_value0)
+        fill_value1 = "bar"
+        ds.createVariable("x1", str, ("string",), fill_value=fill_value1)
+
+    # check if new API can read them
+    with h5netcdf.File(tmp_local_netcdf, "r", **decode_vlen_strings) as ds:
+        decode_vlen = decode_vlen_strings["decode_vlen_strings"]
+        fvalue0 = fill_value0 if decode_vlen else fill_value0.encode("utf-8")
+        fvalue1 = fill_value1 if decode_vlen else fill_value1.encode("utf-8")
+        assert ds["x0"][0] == fvalue0
+        assert ds["x0"].attrs["_FillValue"] == fill_value0
+        assert ds["x1"][0] == fvalue1
+        assert ds["x1"].attrs["_FillValue"] == fill_value1
+
+    # check if legacyapi can read them
+    with legacyapi.Dataset(tmp_local_netcdf, "r") as ds:
+        assert ds["x0"][0] == fill_value0
+        assert ds["x0"]._FillValue == fill_value0
+        assert ds["x1"][0] == fill_value1
+        assert ds["x1"]._FillValue == fill_value1
+
+    # check if netCDF4-python can read them
+    with netCDF4.Dataset(tmp_local_netcdf, "r") as ds:
+        assert ds["x0"][0] == fill_value0
+        assert ds["x0"]._FillValue == fill_value0
+        assert ds["x1"][0] == fill_value1
+        assert ds["x1"]._FillValue == fill_value1
